"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryTransformPipe = void 0;
const tslib_1 = require("tslib");
const types_1 = require("../../../types");
const helper_1 = require("../../../helper");
const constants_1 = require("../../../constants");
function convertToFilterObject(value) {
    if ((0, helper_1.isString)(value)) {
        return {
            [types_1.FilterOperand.eq]: value,
        };
    }
    else {
        const arrayOp = {
            [types_1.FilterOperand.in]: true,
            [types_1.FilterOperand.nin]: true,
            [types_1.FilterOperand.some]: true,
        };
        return Object.entries(value).reduce((acum, [op, filed]) => {
            if (arrayOp[op]) {
                acum[op] = ((0, helper_1.isString)(filed) ? filed.split(',') : []).filter((i) => !!i);
            }
            else {
                acum[op] = filed;
            }
            return acum;
        }, {});
    }
}
class QueryTransformPipe {
    constructor(repository) {
        this.repository = repository;
    }
    transform(value) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const builtQuery = {
                ["sort" /* QueryField.sort */]: null,
                ["filter" /* QueryField.filter */]: {
                    relation: null,
                    target: null,
                },
                ["include" /* QueryField.include */]: null,
                ["page" /* QueryField.page */]: {
                    number: constants_1.DEFAULT_QUERY_PAGE,
                    size: constants_1.DEFAULT_PAGE_SIZE,
                },
                ["fields" /* QueryField.fields */]: null,
                ["needAttribute" /* QueryField.needAttribute */]: false,
            };
            if (!value) {
                return builtQuery;
            }
            builtQuery['needAttribute'] = !!value['needAttribute'];
            if (value["filter" /* QueryField.filter */]) {
                builtQuery['filter'] = Object.entries(value["filter" /* QueryField.filter */]).reduce((accum, [field, value]) => {
                    const objectOperand = convertToFilterObject(value);
                    if (Object.keys(objectOperand).length === 0) {
                        return accum;
                    }
                    if (field.indexOf('.') > -1) {
                        const [relation, fieldRelation] = field.split('.');
                        accum['relation'] = !accum['relation'] ? {} : accum['relation'];
                        accum['relation'][relation] = accum['relation'][relation] || {};
                        accum['relation'][relation][fieldRelation] = objectOperand;
                    }
                    else {
                        accum['target'] = !accum['target'] ? {} : accum['target'];
                        accum['target'][field] = objectOperand;
                    }
                    return accum;
                }, { relation: null, target: null });
            }
            if (value["include" /* QueryField.include */] &&
                (0, helper_1.isString)(value["include" /* QueryField.include */])) {
                builtQuery["include" /* QueryField.include */] = value["include" /* QueryField.include */]
                    .split(',')
                    .map((i) => i.trim())
                    .filter((i) => !!i);
            }
            if (value["fields" /* QueryField.fields */]) {
                builtQuery["fields" /* QueryField.fields */] = Object.entries(value["fields" /* QueryField.fields */]).reduce((acum, [key, value]) => {
                    acum[key] = value
                        .split(',')
                        .map((i) => i.trim())
                        .filter((i) => !!i);
                    return acum;
                }, {});
            }
            if (value["sort" /* QueryField.sort */] && (0, helper_1.isString)(value["sort" /* QueryField.sort */])) {
                builtQuery["sort" /* QueryField.sort */] = value["sort" /* QueryField.sort */]
                    .split(',')
                    .map((i) => i.trim())
                    .filter((i) => !!i)
                    .reduce((acum, field) => {
                    const fieldName = field.charAt(0) === '-' ? field.substring(1) : field;
                    const sort = field.charAt(0) === '-' ? 'DESC' : 'ASC';
                    if (fieldName.indexOf('.') > -1) {
                        const [relation, fieldRelation] = field.split('.');
                        const relationName = relation.charAt(0) === '-' ? relation.substring(1) : relation;
                        acum[relationName] = acum[relation] || {};
                        acum[relationName][fieldRelation] = sort;
                    }
                    else {
                        acum['target'] = acum['target'] || {};
                        acum['target'][fieldName] = sort;
                    }
                    return acum;
                }, {});
            }
            if (value["page" /* QueryField.page */]) {
                const { number, size } = value["page" /* QueryField.page */];
                const sizeParse = parseInt(size, 10);
                const numberParse = parseInt(number, 10);
                if (number && !isNaN(numberParse)) {
                    builtQuery["page" /* QueryField.page */].number = numberParse;
                }
                if (size && !isNaN(sizeParse)) {
                    builtQuery["page" /* QueryField.page */].size = sizeParse;
                }
            }
            return builtQuery;
        });
    }
}
exports.QueryTransformPipe = QueryTransformPipe;
//# sourceMappingURL=query-transform.pipe.js.map