"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UtilsMethode = void 0;
const tslib_1 = require("tslib");
const typeorm_1 = require("typeorm");
const common_1 = require("@nestjs/common");
const types_1 = require("../../../../types");
const helper_1 = require("../../../../helper");
const types_common_1 = require("../../../../types-common");
class UtilsMethode {
    static getParamName(field, i) {
        return `params_${field}_${i}`;
    }
    static applyQueryFiltersTarget(builder, filter, metadata) {
        const resultExpression = [];
        const preparedResourceName = (0, helper_1.snakeToCamel)(metadata.name);
        if (!UtilsMethode.relationNameMapByEntity.has(preparedResourceName)) {
            UtilsMethode.relationNameMapByEntity.set(preparedResourceName, new Set(metadata.relations.map((item) => item.propertyName)));
        }
        const relations = UtilsMethode.relationNameMapByEntity.get(preparedResourceName);
        let field;
        let i = 0;
        for (field in filter) {
            const operand = Object.keys(filter[field]).pop();
            const value = operand === types_1.FilterOperand.like
                ? `%${filter[field][operand]}%`
                : filter[field][operand];
            if (relations.has(field)) {
                const relation = metadata.relations.find((item) => item.propertyName === field);
                const { inverseSidePropertyPath, inverseEntityMetadata: { target, name }, } = relation;
                const resourceRelationName = (0, helper_1.snakeToCamel)(name);
                const primaryColumn = metadata.primaryColumns[0].databaseName;
                switch (relation.relationType) {
                    case 'many-to-many': {
                        const { inverseJoinColumns, joinColumns } = relation.isManyToManyOwner ? relation : relation.inverseRelation;
                        const relationProps = relation.isManyToManyOwner
                            ? relation
                            : relation.inverseRelation;
                        const { joinTableName } = relationProps;
                        const { databaseName: queryJoinPropsName } = relation.isManyToManyOwner
                            ? inverseJoinColumns[0]
                            : joinColumns[0];
                        const { databaseName: selectJoinPropsName } = relation.isManyToManyOwner
                            ? joinColumns[0]
                            : inverseJoinColumns[0];
                        const onQuery = `${joinTableName}.${queryJoinPropsName} = ${resourceRelationName}.${primaryColumn}`;
                        const selectQuery = `${joinTableName}.${selectJoinPropsName}`;
                        const query = builder
                            .subQuery()
                            .select(selectQuery)
                            .from(joinTableName, joinTableName)
                            .leftJoin(resourceRelationName, resourceRelationName, onQuery)
                            .where(`${selectQuery} = ${builder.alias}.${primaryColumn}`)
                            .getQuery();
                        resultExpression.push({
                            expression: types_common_1.OperandMapForNullRelation[operand].replace('EXPRESSION', query),
                            params: null,
                        });
                        break;
                    }
                    case 'one-to-many': {
                        const query = builder
                            .subQuery()
                            .select(`${resourceRelationName}.${inverseSidePropertyPath}`)
                            .from(target, resourceRelationName)
                            .where(`${resourceRelationName}.${inverseSidePropertyPath} = ${builder.alias}.id`)
                            .getQuery();
                        resultExpression.push({
                            expression: types_common_1.OperandMapForNullRelation[operand].replace('EXPRESSION', query),
                            params: null,
                        });
                        break;
                    }
                    default: {
                        resultExpression.push({
                            expression: `${preparedResourceName}.${field.toString()} ${types_common_1.OperandMapForNull[operand].replace('EXPRESSION', UtilsMethode.getParamName(`${preparedResourceName}.${field}`, i))}`,
                            params: null,
                        });
                    }
                }
            }
            else {
                resultExpression.push({
                    expression: `${preparedResourceName}.${field.toString()} ${types_common_1.OperandsMap[operand].replace('EXPRESSION', UtilsMethode.getParamName(`${preparedResourceName}.${field}`, i))}`,
                    params: {
                        name: UtilsMethode.getParamName(`${preparedResourceName}.${field}`, i),
                        val: value,
                    },
                });
            }
            i++;
        }
        return resultExpression;
    }
    static applyQueryFilterRelation(builder, filter, metadata) {
        const resultExpression = [];
        const preparedResourceName = (0, helper_1.snakeToCamel)(metadata.name);
        const primaryColumn = metadata.primaryColumns[0].databaseName;
        let relationProperty;
        let i = 0;
        for (relationProperty in filter) {
            if (!filter[relationProperty]) {
                continue;
            }
            const relation = metadata.relations.find((item) => {
                return item.propertyName === relationProperty;
            });
            const { inverseSidePropertyPath, inverseEntityMetadata: { target, name }, } = relation;
            const resourceName = (0, helper_1.snakeToCamel)(name);
            let relationFieldProperty;
            for (relationFieldProperty in filter[relationProperty]) {
                const operand = Object.keys(filter[relationProperty][relationFieldProperty]).pop();
                const value = operand === types_1.FilterOperand.like
                    ? `%${filter[relationProperty][relationFieldProperty][operand]}%`
                    : filter[relationProperty][relationFieldProperty][operand];
                const currentOperandMap = value.toString().toLocaleLowerCase() === 'null'
                    ? types_common_1.OperandMapForNull
                    : types_common_1.OperandsMap;
                const paramsField = value.toString().toLocaleLowerCase() === 'null'
                    ? null
                    : UtilsMethode.getParamName(`${relationProperty}.${relationFieldProperty.toString()}`, i);
                switch (relation.relationType) {
                    case 'many-to-many': {
                        const { inverseJoinColumns, joinColumns } = relation.isManyToManyOwner ? relation : relation.inverseRelation;
                        const relationProps = relation.isManyToManyOwner
                            ? relation
                            : relation.inverseRelation;
                        const { joinTableName } = relationProps;
                        const { databaseName: queryJoinPropsName } = relation.isManyToManyOwner
                            ? inverseJoinColumns[0]
                            : joinColumns[0];
                        const { databaseName: selectJoinPropsName } = relation.isManyToManyOwner
                            ? joinColumns[0]
                            : inverseJoinColumns[0];
                        const onQuery = `${joinTableName}.${queryJoinPropsName} = ${resourceName}.${primaryColumn}`;
                        const selectQuery = `${joinTableName}.${selectJoinPropsName}`;
                        const query = builder
                            .subQuery()
                            .select(selectQuery)
                            .from(joinTableName, joinTableName)
                            .leftJoin(resourceName, relationProperty, onQuery)
                            .where(`${relationProperty}.${relationFieldProperty.toString()} ${currentOperandMap[operand].replace('EXPRESSION', paramsField)}`)
                            .getQuery();
                        resultExpression.push({
                            expression: `${preparedResourceName}.id IN ${query}`,
                            params: paramsField === null
                                ? null
                                : {
                                    val: value,
                                    name: paramsField,
                                },
                        });
                        break;
                    }
                    case 'one-to-many': {
                        const query = builder
                            .subQuery()
                            .select(`${resourceName}.${inverseSidePropertyPath}`)
                            .from(target, resourceName)
                            .where(`${resourceName}.${relationFieldProperty.toString()} ${currentOperandMap[operand].replace('EXPRESSION', paramsField)}`)
                            .getQuery();
                        resultExpression.push({
                            expression: `${preparedResourceName}.id IN ${query}`,
                            params: paramsField === null
                                ? null
                                : {
                                    val: value,
                                    name: paramsField,
                                },
                        });
                        break;
                    }
                    default:
                        resultExpression.push({
                            expression: `${relationProperty}.${relationFieldProperty.toString()} ${currentOperandMap[operand].replace('EXPRESSION', paramsField)}`,
                            params: paramsField === null
                                ? null
                                : {
                                    val: value,
                                    name: paramsField,
                                },
                            selectInclude: relationProperty,
                        });
                        break;
                }
                i++;
            }
        }
        return resultExpression;
    }
    static asyncIterateFindRelationships(relationships, repository) {
        return tslib_1.__asyncGenerator(this, arguments, function* asyncIterateFindRelationships_1() {
            const relationsKeys = Object.keys(relationships);
            const countRelationsKeys = relationsKeys.length;
            const preparedResourceName = (0, helper_1.snakeToCamel)(repository.metadata.name);
            if (!UtilsMethode.relationNameMapByEntity.has(preparedResourceName)) {
                UtilsMethode.relationNameMapByEntity.set(preparedResourceName, new Set(repository.metadata.relations.map((item) => item.propertyName)));
            }
            const relations = UtilsMethode.relationNameMapByEntity.get(preparedResourceName);
            let i = 0;
            while (countRelationsKeys > i) {
                const relationsName = relationsKeys[i];
                if (!relations.has(relationsKeys[i])) {
                    throw new common_1.NotFoundException({
                        detail: `Resource for props '${relationsName}' does not exist`,
                    });
                }
                const { data } = relationships[relationsName];
                if (data === null) {
                    yield yield tslib_1.__await({ propsName: relationsName, type: null, rel: null, id: null });
                    i++;
                    continue;
                }
                const isArray = Array.isArray(data);
                if (isArray && data.length === 0) {
                    yield yield tslib_1.__await({ propsName: relationsName, type: null, rel: [], id: null });
                    i++;
                    continue;
                }
                const condition = isArray ? (0, typeorm_1.In)(data.map((i) => i.id)) : (0, typeorm_1.Equal)(data['id']);
                const relationsTypeName = isArray ? data[0]['type'] : data['type'];
                const result = yield tslib_1.__await(repository.manager
                    .getRepository(relationsTypeName)
                    .find({
                    select: {
                        id: true,
                    },
                    where: {
                        id: condition,
                    },
                }));
                const relData = !isArray ? [data] : data;
                const idsToAdd = relData.map((i) => i.id);
                if ((isArray &&
                    (result.length === 0 || result.length !== idsToAdd.length)) ||
                    (!isArray && result.length === 0)) {
                    const detail = isArray
                        ? `Resource '${relationsTypeName}' with ids '${idsToAdd.join(',')}' does not exist`
                        : `Resource '${relationsTypeName}' with id '${idsToAdd[0]}' does not exist`;
                    throw new common_1.NotFoundException({
                        detail,
                    });
                }
                const { type } = relData[0];
                yield yield tslib_1.__await({
                    propsName: relationsName,
                    type,
                    rel: isArray ? result : result[0],
                    id: isArray ? idsToAdd : idsToAdd[0],
                });
                i++;
            }
        });
    }
    static validateRelationRequestData(repository, id, relName, body) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const relationItem = repository.metadata.relations.find((i) => i.propertyName === relName);
            const typeName = (0, helper_1.camelToKebab)((0, helper_1.getEntityName)(relationItem.inverseEntityMetadata.target));
            const isArray = Array.isArray(body);
            if (['one-to-many', 'many-to-many'].includes(relationItem.relationType) &&
                !isArray) {
                throw new common_1.UnprocessableEntityException([
                    {
                        source: {
                            parameter: '/data',
                        },
                        detail: `Body data should be array`,
                    },
                ]);
            }
            if (['one-to-one', 'many-to-one'].includes(relationItem.relationType) &&
                isArray) {
                throw new common_1.UnprocessableEntityException([
                    {
                        source: {
                            parameter: '/data',
                        },
                        detail: `Body data should be object`,
                    },
                ]);
            }
            const findOneOptions = {
                select: {
                    [repository.metadata.primaryColumns[0].propertyName]: true,
                },
                where: {
                    id: (0, typeorm_1.Equal)(id),
                },
            };
            const item = yield repository.findOne(findOneOptions);
            if (!item) {
                throw new common_1.NotFoundException([
                    {
                        detail: `Not exist item "${id}" in resource "${(0, helper_1.camelToKebab)((0, helper_1.getEntityName)(repository.target))}"`,
                    },
                ]);
            }
            if (body === null || (isArray && body.length === 0)) {
                return {
                    item,
                    rel: isArray ? [] : null,
                    relationItem,
                };
            }
            const prepareData = isArray ? body : [body];
            const errors = [];
            for (const prepareItem in prepareData) {
                if (prepareData[prepareItem]['type'] !== typeName) {
                    errors.push({
                        source: {
                            parameter: isArray ? `/data/${prepareItem}` : '/data',
                        },
                        detail: `Type should be equal type of relName: "${relName}". Type of "${relName}" is ${typeName}`,
                    });
                }
            }
            if (errors.length) {
                throw new common_1.UnprocessableEntityException(errors);
            }
            const relIds = prepareData.map((i) => i.id);
            const relData = yield repository.manager
                .getRepository(relationItem.inverseEntityMetadata.target)
                .find({
                select: {
                    [relationItem.inverseEntityMetadata.primaryColumns[0].propertyName]: true,
                },
                where: {
                    id: (0, typeorm_1.In)(relIds),
                },
            });
            if (relData.length !== prepareData.length) {
                const mapRelData = relData.reduce((acum, item) => {
                    acum[item.id] = true;
                    return acum;
                }, {});
                throw new common_1.NotFoundException(relIds
                    .filter((id) => !mapRelData[id])
                    .map((id) => ({
                    detail: `Not exist item "${id}" in relation "${relName}"`,
                })));
            }
            return {
                item,
                rel: isArray ? relData : relData[0],
                relationItem,
            };
        });
    }
}
exports.UtilsMethode = UtilsMethode;
UtilsMethode.relationNameMapByEntity = new Map();
//# sourceMappingURL=utils-methode.js.map